# Exemplu utilizare pachet CARET 
library("caret")

dataSet <- iris[,1:4]
dataClass <- iris[,5]

# Etape de pre-procesare

# 1. Eliminarea atributelor corelate (functie CARET:  findCorrelation)

# Calculul matricii de corelatie
dataCorr <- cor(dataSet)  

# Determinarea atributelor cu coeficient de corelatie > 0.9
highCorr <- findCorrelation(dataCorr, 0.90) 

# Eliminarea atributelor corelate
dataUncorr <- dataSet[, -highCorr]

dataSet <- dataUncorr

# 2. Transformarea datelor (scalare bazata pe domeniu, centrare, standardizare, transformare PCA etc)

transformCenter <- preProcess(dataSet, method="center") # definirea transformarii
dataCenter <- predict(transformCenter, dataSet)         # aplicarea transformarii   

# Alte transformari: "scale" (scalare), 
# "pca" (proiectie pe componentele principale), 
# "spatialSign" (normalizare)
transformPCA <- preProcess(dataSet, method="pca", thresh=0.9) # se aleg componentele care conserva 90% din variabilitatea datelor
dataPCA <- predict(transformPCA, dataSet) # atribute noi: PC1, PC2, ...

# 3. Construirea setului de antrenare si a setului de testare - selectie stratificata (stratified sampling) 
# Functie CARET:  createDataPartition

# initializare generator de numere aleatoare - utila pt asigurarea replicabilitatii rezultatelor
set.seed(1)

# Partitionare set de date in set antrenare (75%) si set testare (25%) 
# a.i. distributia claselor sa fie conservata (stratified sampling)
# dataClass = atributul care specifica clasa 
inTrain <- createDataPartition(dataClass, p = 0.75, list = FALSE) # indici date set antrenare

# separarea setului de date in trainingSet si testingSet
trainingSet <- dataSet[inTrain,]
testingSet <- dataSet[-inTrain,]

# distributia claselor in cele doua seturi (doar pt a verifica conservarea distributiei)
trainingClass <- dataClass[inTrain]
testingClass <- dataClass[-inTrain]

prop.table(table(dataClass))

prop.table(table(trainingClass))

prop.table(table(testingClass))

# ??? Ce se poate observa?   
# Comparati rezultatul obtinut folosind "createDataPartition" cu cel obtinut folosind "sample" 

sizeTrain <- 0.75*length(dataClass)
trainSample <- sample(dataClass, sizeTrain, replace = FALSE)
prop.table(table(trainSample))


# 4. Antrenarea modelului

# setarea parametrilor de control 
# (se folosesc valori implicite, cu exceptia numarului de etape de selectie in bootstrapping care este setat la 100)
bootControl <- trainControl(number = 100)

# antrenare model SVM 
svmFit <- train(trainingSet, trainingClass, method = "svmRadial", tuneLength = 5, trControl = bootControl, scaled = FALSE)

# analiza rezultatelor antrenarii
svmFit

# analiza modelului selectat (selectie bazata pe acuratetea si complexitatea modelului 

svmFit$finalModel

# vizualizarea masurii de acuratete vs parametri (pt SVM e utilizat doar parametrul C)
plot(svmFit)

# similar pt masura kappa
plot(svmFit, metric = "Kappa")

# grafic distributie de probabilitate (aproximata prin histograme) corespunzatoare masurilor de acuratete si Kappa 
# bazate pe rezultate obtinute pentru date de testare selectate aleatoar
resampleHist(svmFit)

# 5. Evaluarea modelului

# predictia clasei pentru date noi (din setul de testare)
resTesting <- predict(svmFit, testingSet) # type="prob" - necesar pt ROC

# compararea rezultatelor prezise cu cele adevarate 
predict(svmFit, testingSet)==testingClass

# construirea matricii de confuzie
confusionMatrix(data = resTesting, reference = testingClass)


